from datetime import timezone
import os
import uuid
import qrcode
from chat_app.models import ChatRoom
from crea_app.models import CustomUser
from social_app.models import Block, Follow, Report
from save_app.models import SavedArtist
from rest_framework.decorators import action
from django.conf import settings
from django.core.files.storage import FileSystemStorage
from django.contrib.auth import get_user_model
from rest_framework import viewsets, status
from rest_framework.response import Response
from rest_framework.exceptions import ValidationError
from .models import ArtiztNetworkAccNumber, City, ConfirmKYC, Language, Profile, ProfileMember, ProfileView, Role, Industry, Skill, Experience, Education, State, StateShortNames, UnionAssociation, UserLanguage
from .serializers import AadharVerificationSerializer, CitySerializer, ConfirmKYCDocumentSerializer, DLVerificationSerializer, DocumentUploadSerializer, ExperienceSerializer, EducationSerializer, LanguageSerializer, PassportVerificationSerializer, ProfileCompletionStatusSerializer, ProfileCreateSerializer, ProfileMemberSerializer, ProfileSerializer, RoleSerializer, IndustrySerializer, SkillSerializer, StateSerializer, StateShortNamesSerializer, UnionAssociationSerializer, UserLanguageSerializer
from rest_framework.permissions import AllowAny, IsAuthenticated
from crea_app.storages import UthoStorage
from rest_framework.views import APIView
from django.core.exceptions import ObjectDoesNotExist  
import logging
from django.utils import timezone
from django.db.models import Q
from django.http import JsonResponse
from django.shortcuts import get_object_or_404
from rest_framework.parsers import MultiPartParser, FormParser

User = get_user_model()

def get_profile(user_id):
    user = User.objects.filter(id=user_id).first()
    if not user:
        return None
    profile, _ = Profile.objects.get_or_create(user=user)
    return profile


class RoleListView(viewsets.ViewSet):
    permission_classes = [AllowAny]
    
    def list(self, request):
        roles = Role.objects.all()
        if not roles.exists():
            return Response({"status": "error", "message": "No roles available"}, status=status.HTTP_404_NOT_FOUND)

        serializer = RoleSerializer(roles, many=True)
        return Response({"status": "success", "data": serializer.data}, status=status.HTTP_200_OK)

class RoleSelectionView(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]
    
    def create(self, request):
        user_id = request.data.get('user_id')
        role_id = request.data.get('role_id')

        if not user_id or not role_id:
            return Response({"error": "user_id and role_id are required fields."}, status=status.HTTP_400_BAD_REQUEST)

        user = User.objects.filter(id=user_id).first()
        if not user:
            return Response({"error": "User not found"}, status=status.HTTP_404_NOT_FOUND)

        role = Role.objects.filter(id=role_id).first()
        if not role:
            return Response({"error": "Role not found"}, status=status.HTTP_404_NOT_FOUND)

        profile, _ = Profile.objects.get_or_create(user=user)
        profile.selected_role = role
        profile.save()

        return Response({"status": "success", "message": "Role selected successfully", "data": {"role": role.role_name, "id": role.id}}, status=status.HTTP_200_OK)

    def update(self, request, user_id=None):
        new_role_id = request.data.get('role_id')

        if not user_id or not new_role_id:
            return Response({"error": "user_id and role_id are required fields."}, status=status.HTTP_400_BAD_REQUEST)

        user = User.objects.filter(id=user_id).first()
        if not user:
            return Response({"error": "User not found"}, status=status.HTTP_404_NOT_FOUND)

        new_role = Role.objects.filter(id=new_role_id).first()
        if not new_role:
            return Response({"error": "New role not found"}, status=status.HTTP_404_NOT_FOUND)

        profile = Profile.objects.filter(user=user).first()
        if not profile:
            return Response({"error": "Profile not found for this user"}, status=status.HTTP_404_NOT_FOUND)

        profile.selected_role = new_role
        profile.save()

        return Response({"status": "success", "message": "Role updated successfully", "data": {"role": new_role.role_name, "id": new_role.id}}, status=status.HTTP_200_OK)

class IndustryListView(viewsets.ViewSet):
    permission_classes = [AllowAny]
    
    def list(self, request):
        industries = Industry.objects.all()
        if not industries.exists():
            return Response({"status": "error", "message": "No industries found"}, status=status.HTTP_404_NOT_FOUND)

        serializer = IndustrySerializer(industries, many=True)
        return Response({"status": "success", "data": serializer.data}, status=status.HTTP_200_OK)

class IndustrySelectionView(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]
    
    def create(self, request):
        user_id = request.data.get('user_id')
        industry_ids = request.data.get('industry_ids', [])

        if not user_id or not industry_ids:
            return Response({"error": "user_id and industry_ids are required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        industries = Industry.objects.filter(id__in=industry_ids)
        if industries.count() != len(industry_ids):
            return Response({"error": "Some industries are invalid."}, status=status.HTTP_400_BAD_REQUEST)

        profile.selected_industries.set(industries)
        profile.save()

        return Response({"status": "success", "message": "Industries selected successfully", "data": {"selected_industries": industry_ids}}, status=status.HTTP_200_OK)

    def update(self, request, user_id=None):
        industry_ids = request.data.get('industry_ids', [])

        if not user_id or not industry_ids:
            return Response({"error": "user_id and industry_ids are required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        industries = Industry.objects.filter(id__in=industry_ids)
        if industries.count() != len(industry_ids):
            return Response({"error": "Some industries are invalid."}, status=status.HTTP_400_BAD_REQUEST)

        profile.selected_industries.set(industries)
        profile.save()

        return Response({"status": "success", "message": "Industries updated successfully", "data": {"selected_industries": industry_ids}}, status=status.HTTP_200_OK)

    def destroy(self, request, *args, **kwargs):
        user_id = request.user.id
        industry_id = kwargs.get('industry_id')
        if not user_id:
            return Response({"error": "user_id is required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        try:
            industry = Industry.objects.get(id=industry_id)
        except Skill.DoesNotExist:
            return Response({"error": "Industry not found."}, status=status.HTTP_404_NOT_FOUND)

        if industry in profile.selected_industries.all():
            profile.selected_industries.remove(industry)
            profile.save()
            return Response({"status": "success", "message": "Industry removed successfully"}, status=status.HTTP_200_OK)
        else:
            return Response({"error": "Industry not associated with this profile."}, status=status.HTTP_400_BAD_REQUEST)

        # return Response({"status": "success", "message": "Industries removed successfully"}, status=status.HTTP_200_OK)

    def list_selected(self, request, user_id=None):
        if not user_id:
            return Response({"error": "user_id is required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        selected_industries = profile.selected_industries.all()
        serializer = IndustrySerializer(selected_industries, many=True)
        
        return Response({"status": "success", "data": serializer.data}, status=status.HTTP_200_OK)

class PrimaryIndustrySelectionView(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]
    
    def create(self, request):
        user_id = request.data.get('user_id')
        primary_industry_id = request.data.get('primary_industry_id')

        if not user_id or not primary_industry_id:
            return Response({"error": "user_id and primary_industry_id are required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        primary_industry = Industry.objects.filter(id=primary_industry_id).first()
        if not primary_industry or primary_industry not in profile.selected_industries.all():
            return Response({"error": "Primary industry must be chosen from selected industries."}, status=status.HTTP_400_BAD_REQUEST)

        profile.selected_primary_industry = primary_industry
        profile.save()

        return Response({"status": "success", "message": "Primary industry selected successfully", "data": {"primary_industry": primary_industry.name}}, status=status.HTTP_200_OK)

    def update(self, request, user_id=None):
        primary_industry_id = request.data.get('primary_industry_id')

        if not user_id or not primary_industry_id:
            return Response({"error": "user_id and primary_industry_id are required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        primary_industry = Industry.objects.filter(id=primary_industry_id).first()
        if not primary_industry or primary_industry not in profile.selected_industries.all():
            return Response({"error": "Primary industry must be chosen from selected industries."}, status=status.HTTP_400_BAD_REQUEST)

        profile.selected_primary_industry = primary_industry
        profile.save()

        return Response({"status": "success", "message": "Primary industry updated successfully", "data": {"primary_industry": primary_industry.name}}, status=status.HTTP_200_OK)

    def destroy(self, request, user_id=None):
        if not user_id:
            return Response({"error": "user_id is required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        profile.selected_primary_industry = None
        profile.save()

        return Response({"status": "success", "message": "Primary industry removed successfully"}, status=status.HTTP_200_OK)

class SkillListView(viewsets.ViewSet):
    permission_classes = [AllowAny]
    
    def list(self, request):
        skills = Skill.objects.all()
        if not skills.exists():
            return Response({"status": "error", "message": "No skills found"}, status=status.HTTP_404_NOT_FOUND)

        serializer = SkillSerializer(skills, many=True)
        return Response({"status": "success", "data": serializer.data}, status=status.HTTP_200_OK)

class SkillFilterAPIView(APIView):
    permission_classes = [AllowAny]

    def post(self, request):
        industry_ids = request.data.get('industry_ids', [])
        if not industry_ids or not isinstance(industry_ids, list):
            return Response(
                {"status": "error", "message": "Please provide a valid list of industry IDs"},
                status=status.HTTP_400_BAD_REQUEST
            )

        # Fetch skills associated with the provided industry IDs
        skills = Skill.objects.filter(industry_id__in=industry_ids)
        if not skills.exists():
            return Response(
                {"status": "error", "message": "No skills found for the provided industry IDs"},
                status=status.HTTP_404_NOT_FOUND
            )

        serializer = SkillSerializer(skills, many=True)
        return Response(
            {"status": "success", "data": serializer.data},
            status=status.HTTP_200_OK
        )
    
class SkillSelectionView(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]
    
    def create(self, request):
        user_id = request.data.get('user_id')
        skill_ids = request.data.get('skill_ids', [])

        if not user_id or not skill_ids:
            return Response({"error": "user_id and skill_ids are required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        skills = Skill.objects.filter(id__in=skill_ids)
        if skills.count() != len(skill_ids):
            return Response({"error": "Some skills are invalid."}, status=status.HTTP_400_BAD_REQUEST)

        profile.selected_skills.set(skills)
        profile.save()

        return Response({"status": "success", "message": "Skills selected successfully", "data": {"selected_skills": skill_ids}}, status=status.HTTP_200_OK)

    # List selected skills for a user
    def list_selected(self, request, user_id=None):
        profile = get_profile(user_id)
        if not profile:
            return Response({"status": "error", "message": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        selected_skills = profile.selected_skills.all()
        if not selected_skills:
            return Response({"status": "error", "message": "No selected skills found."}, status=status.HTTP_404_NOT_FOUND)

        serializer = SkillSerializer(selected_skills, many=True)
        return Response({"status": "success", "data": serializer.data}, status=status.HTTP_200_OK)

    # Update selected skills for a user
    def update(self, request, user_id=None):
        skill_ids = request.data.get('skill_ids', [])

        if not user_id or not skill_ids:
            return Response({"error": "user_id and skill_ids are required for update."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        skills = Skill.objects.filter(id__in=skill_ids)
        if skills.count() != len(skill_ids):
            return Response({"error": "Some skills are invalid."}, status=status.HTTP_400_BAD_REQUEST)

        profile.selected_skills.set(skills)
        profile.save()

        return Response({"status": "success", "message": "Skills updated successfully", "data": {"selected_skills": skill_ids}}, status=status.HTTP_200_OK)

    # Remove all selected skills for a user
    def destroy(self, request, *args, **kwargs):
        user_id = request.user.id
        skill_id = kwargs.get('skill_id')
        if not user_id:
            return Response({"error": "user_id is required to delete skills."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        try:
            print(skill_id)
            skill = Skill.objects.get(id=skill_id)
        except Skill.DoesNotExist:
            return Response({"error": "Skill not found."}, status=status.HTTP_404_NOT_FOUND)

        if skill in profile.selected_skills.all():
            profile.selected_skills.remove(skill)
            profile.save()
            return Response({"status": "success", "message": "Skill removed successfully"}, status=status.HTTP_200_OK)
        else:
            return Response({"error": "Skill not associated with this profile."}, status=status.HTTP_400_BAD_REQUEST)

        # return Response({"status": "success", "message": "All selected skills removed successfully"}, status=status.HTTP_200_OK)

class PrimarySkillSelectionView(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]
    
    def create(self, request):
        user_id = request.data.get('user_id')
        primary_skill_id = request.data.get('primary_skill_id')

        if not user_id or not primary_skill_id:
            return Response({"error": "user_id and primary_skill_id are required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        primary_skill = Skill.objects.filter(id=primary_skill_id).first()
        if not primary_skill or primary_skill not in profile.selected_skills.all():
            return Response({"error": "Primary skill must be chosen from selected skills."}, status=status.HTTP_400_BAD_REQUEST)

        profile.selected_primary_skill = primary_skill
        profile.save()

        return Response({"status": "success", "message": "Primary skill selected successfully", "data": {"primary_skill": primary_skill.name}}, status=status.HTTP_200_OK)

    def update(self, request, user_id=None):
        primary_skill_id = request.data.get('primary_skill_id')

        if not user_id or not primary_skill_id:
            return Response({"error": "user_id and primary_skill_id are required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        primary_skill = Skill.objects.filter(id=primary_skill_id).first()
        if not primary_skill or primary_skill not in profile.selected_skills.all():
            return Response({"error": "Primary skill must be chosen from selected skills."}, status=status.HTTP_400_BAD_REQUEST)

        profile.selected_primary_skill = primary_skill
        profile.save()

        return Response({"status": "success", "message": "Primary skill updated successfully", "data": {"primary_skill": primary_skill.name}}, status=status.HTTP_200_OK)

    def destroy(self, request, user_id=None):
        if not user_id:
            return Response({"error": "user_id is required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"error": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        profile.selected_primary_skill = None
        profile.save()

        return Response({"status": "success", "message": "Primary skill removed successfully"}, status=status.HTTP_200_OK)


class ProfileViewSet(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]
    
    def create(self, request):
        user_id = request.data.get('user_id')
        
        if not user_id:
            return Response({"status": "error", "message": "user_id is required."}, status=status.HTTP_400_BAD_REQUEST)

        profile = get_profile(user_id)
        if not profile:
            return Response({"status": "error", "message": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        # Prepare data dictionary for partial update
        data = {
            "user_type": request.data.get('user_type', profile.user_type or "2"),
            "bio": request.data.get('bio', profile.bio),
            "date_of_birth": request.data.get('date_of_birth', profile.date_of_birth),
            "age": request.data.get('age', profile.age),
            "gender": request.data.get('gender', profile.gender),
            "location": request.data.get('location', profile.location),
            "height": request.data.get('height', profile.height),
            "weight": request.data.get('weight', profile.weight),
        }

        # Only include file fields if they're valid files in the request data
        if 'cover_image' in request.FILES:
            data['cover_image'] = request.FILES['cover_image']
        elif 'cover_image' in request.data and request.data.get('cover_image') is None:
            data['cover_image'] = None
        
        if 'profile_image' in request.FILES:
            data['profile_image'] = request.FILES['profile_image']
        elif 'profile_image' in request.data and request.data.get('profile_image') is None:
            data['profile_image'] = None

        serializer = ProfileCreateSerializer(profile, data=data, partial=True)
        print(serializer.is_valid())
        if serializer.is_valid():
            location = request.data.get('location', profile.location)
            print(location)
            if location:
                state_code = location.split(',')[-1].strip()  # Get state code after the comma
                # state_name = State.objects.filter(id=state_code).first()
                state_entry = StateShortNames.objects.filter(id=state_code).first()

                if state_entry:
                    state_code = state_entry.code
                    # Generate ArtiztNetworkAccNumber with the state code
                    if not hasattr(profile.user, 'artizt_account'):
                        current_year = timezone.now().year
                        unique_number = uuid.uuid4().int % 100000
                        account_number = f"CN/IND/{state_code}/{current_year}/{unique_number}"
                        ArtiztNetworkAccNumber.objects.create(user=profile.user, account_number=account_number)

            serializer.save()
            # print(profile.location)
            return Response({
                "status": "success",
                "message": "Profile created successfully",
                "error": False,
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)
        
        return Response({
            "status": "error",
            "message": "Profile creation failed due to invalid data.",
            "error": True,
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)


    def retrieve(self, request, user_id=None):
        profile = get_profile(user_id)
        # print(user_id)
        completion_data = profile.calculate_section_completion()
        if not profile:
            return Response({"status": "error", "message": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        location = None

    # Process location if not null
        if profile.location:
            try:
                # Split the location into city_id and state_id
                location_data = profile.location.split(",")
                # print(location_data)
                if len(location_data) == 2:
                    city_id, state_id = map(int, location_data)  # Convert both parts to integers

                    # Fetch city and state names
                    city = City.objects.filter(id=city_id).first()
                    state = State.objects.filter(id=state_id).first()

                    if city and state:
                        location = f"{city.name}, {state.name}"
                    else:
                        location = None  # If either city or state is not found, set location to None
                else:
                    location = profile.location  # If format is invalid, return the original string
            except (ValueError, TypeError):
                location = profile.location  # If any error occurs, return the original string

        viewer = request.user
        if viewer.is_authenticated and viewer.id != profile.user.id:
            if not ProfileView.objects.filter(profile=profile, viewer=viewer).exists():
                ProfileView.objects.create(profile=profile, viewer=viewer)
                profile.view_count += 1
                profile.save()

        profile = get_profile(user_id)
        artizt_network_acc = ArtiztNetworkAccNumber.objects.filter(user=profile.user).first()
        artiztnetwork_accno = artizt_network_acc.account_number if artizt_network_acc else None
        
        detail = CustomUser.objects.get(id=user_id)
        gender = "Male" if profile.gender == "1" else "Female"
        user_type = "Admin" if profile.user_type == "1" else "Regular User"

        email_id = detail.mobile_or_email
        mobile_number= detail.mobile_number
        full_name= detail.full_name
        serializer = ProfileCreateSerializer(profile)
        completion_percentage = int(serializer.data["completion_percentage"]["totalCompletion"]["percentage"])
        print(completion_percentage)
        if completion_percentage ==100:
            profile_completion_status ="true"
        else:
            profile_completion_status = "false"

        if user_id == request.user.id:
            is_it_my_profile = True
        else:
            is_it_my_profile= False
        print(profile.id)

        save_status = SavedArtist.objects.filter(user_id = request.user.id , artist_profile_id = profile.id ).exists()
        report_status = Report.objects.filter(reporter_id = request.user.id , reported_id = user_id).exists()
        block_status= Block.objects.filter(blocker_id = request.user.id , blocked_id = user_id).exists()

        is_conversation_id_exist = ChatRoom.objects.filter(
                    Q(owner=user_id, members =request.user) | Q(owner=request.user, members=user_id)
                ).exists()


        
        # gender_map = {"1": "Male", "2": "Female"}
        # user_type_map = {"1": "Admin", "2": "Regular User"}

        # gender = gender_map.get(serializer.data.get("gender", "2"), "Female")
        # user_type = user_type_map.get(serializer.data.get("user_type", "2"), "Regular User")
        return Response({
            "status": "success",
            "message": "Profile details retrieved successfully",
            "data": serializer.data,
            "membershiptype": "Artizt Pro",
            "email_id":email_id,
            "mobile_number":mobile_number,
            "location":location,
            "full_name":full_name,
            "artiztnetwork_account_number":artiztnetwork_accno,
            "Completion data":completion_data,
            "Profile_completion_status": profile_completion_status,
            "gender": gender,
            # "profile_visibility":pro
            "user_type": user_type,
            "is_it_my_profile":is_it_my_profile,
            "save_status":save_status,
            "report_status":report_status,
            "block_status":block_status,
            "is_conversation_id_exist":is_conversation_id_exist,
            "profile_url": f"https://app.creanetwork.in/api/profile/{user_id}"
        }, status=status.HTTP_200_OK)

    def partial_update(self, request, user_id=None):
        profile = get_profile(user_id)
        
        if not profile:
            return Response({"status": "error", "message": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        serializer = ProfileCreateSerializer(profile, data=request.data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Profile updated successfully",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        
        return Response({
            "status": "error",
            "message": "Profile update failed due to invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)
        
    def profile_completion(self, request, user_id=None):
        profile = get_profile(user_id)
        
        if not profile:
            return Response({"status": "error", "message": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)

        completion_data = profile.calculate_section_completion()
        serializer = ProfileCompletionStatusSerializer(completion_data)
        
        return Response({
            "status": "success",
            "message": "Profile completion status retrieved successfully",
            "data": serializer.data
        }, status=status.HTTP_200_OK)
        
    @action(detail=True, methods=['get'], url_path='qr-code')
    def generate_qr_code(self, request, pk=None):
        """
        Generates or retrieves a permanent QR code for the user's profile.
        """
        profile = get_profile(pk)
        if not profile:
            return Response({"status": "error", "message": "Profile not found."}, status=status.HTTP_404_NOT_FOUND)
        
        base_url = request.build_absolute_uri('/')[:-1]  # Removes trailing slash
        profile_url = f"{base_url}/api/profile/{profile.user.id}/"

        # profile_url = f"myapp://profile/{profile.user_id}/"  # Use app's custom scheme

        qr_code_filename = f"user_{profile.user.username}_qr.png"
        qr_code_path = f"profiles/{profile.user.id}_{profile.user.username}/qr/{qr_code_filename}"

        storage = UthoStorage()

        # Check if the QR code already exists on Backblaze
        if storage.exists(qr_code_path):
            # Retrieve the QR code URL
            qr_code_url = storage.url(qr_code_path)
        else:
            # Generate the QR code image in memory
            qr_code_image = qrcode.make(profile_url)
            from io import BytesIO
            image_file = BytesIO()
            qr_code_image.save(image_file, format='PNG')
            image_file.seek(0)  # Move cursor to start of file

            # Upload QR code image to Backblaze in the user's folder
            try:
                storage.save(qr_code_path, image_file)
                qr_code_url = storage.url(qr_code_path)
            except Exception as e:
                return Response({
                    "status": "error",
                    "message": "Failed to save QR code.",
                    "details": str(e)
                }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

        return Response({
            "status": "success",
            "user_id": profile.user_id,
            "qr_code_url": qr_code_url
        }, status=status.HTTP_200_OK)

class ExperienceViewSet(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]
    
    def list(self, request, user_id=None):
        if not user_id:
            return Response({"error": "user_id is a required parameter."}, status=status.HTTP_400_BAD_REQUEST)

        experiences = Experience.objects.filter(user_id=user_id)
        if not experiences.exists():
            return Response({
                "status": "error",
                "message": "No experiences found for the specified user.",
                "data": []
            }, status=status.HTTP_404_NOT_FOUND)

        serializer = ExperienceSerializer(experiences, many=True)
        return Response({
            "status": "success",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

    def create(self, request):
        serializer = ExperienceSerializer(data=request.data)
        
        if not request.data.get('user'):
            return Response({"error": "User ID is required to create experience."}, status=status.HTTP_400_BAD_REQUEST)
        
        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Experience added successfully",
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)
        
        return Response({
            "status": "error",
            "message": "Experience creation failed due to invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def update(self, request, pk=None):
        try:
            experience = Experience.objects.get(pk=pk)
        except Experience.DoesNotExist:
            return Response({"error": "Experience not found."}, status=status.HTTP_404_NOT_FOUND)
        
        serializer = ExperienceSerializer(experience, data=request.data, partial=True)
        
        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Experience updated successfully",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        
        return Response({
            "status": "error",
            "message": "Experience update failed due to invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def destroy(self, request, pk=None):
        try:
            experience = Experience.objects.get(pk=pk)
            experience.delete()
            return Response({
                "status": "success",
                "message": "Experience deleted successfully"
            }, status=status.HTTP_200_OK)
        except Experience.DoesNotExist:
            return Response({"error": "Experience not found."}, status=status.HTTP_404_NOT_FOUND)

class EducationViewSet(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]
    
    def list(self, request, user_id=None):
        if not user_id:
            return Response({"error": "user_id is a required parameter."}, status=status.HTTP_400_BAD_REQUEST)

        education = Education.objects.filter(user_id=user_id)
        if not education.exists():
            return Response({
                "status": "error",
                "message": "No education records found for the specified user.",
                "data": []
            }, status=status.HTTP_404_NOT_FOUND)

        serializer = EducationSerializer(education, many=True)
        return Response({
            "status": "success",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

    def create(self, request):
        serializer = EducationSerializer(data=request.data)

        if not request.data.get('user'):
            return Response({"error": "User ID is required to create education entry."}, status=status.HTTP_400_BAD_REQUEST)

        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Education added successfully",
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)
        
        return Response({
            "status": "error",
            "message": "Education creation failed due to invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def update(self, request, pk=None):
        try:
            education = Education.objects.get(pk=pk)
        except Education.DoesNotExist:
            return Response({"error": "Education record not found."}, status=status.HTTP_404_NOT_FOUND)
        
        serializer = EducationSerializer(education, data=request.data, partial=True)
        
        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Education record updated successfully",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        
        return Response({
            "status": "error",
            "message": "Education update failed due to invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def destroy(self, request, pk=None):
        try:
            education = Education.objects.get(pk=pk)
            education.delete()
            return Response({
                "status": "success",
                "message": "Education record deleted successfully"
            }, status=status.HTTP_200_OK)
        except Education.DoesNotExist:
            return Response({"error": "Education record not found."}, status=status.HTTP_404_NOT_FOUND)

class DocumentVerificationViewSet(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]

    def verify_aadhar(self, request):
        serializer = AadharVerificationSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            print(serializer.errors)
            return Response({
                "status": "success",
                "Verified": "success",
                "message": "Aadhar verified successfully."
            }, status=status.HTTP_200_OK)
        return Response({
            "status": "error",
            "message": "Aadhar verification failed",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def verify_passport(self, request):
        serializer = PassportVerificationSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Passport verified successfully."
            }, status=status.HTTP_200_OK)
        return Response({
            "status": "error",
            "message": "Passport verification failed",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def verify_dl(self, request):
        serializer = DLVerificationSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Driving License verified successfully."
            }, status=status.HTTP_200_OK)
        return Response({
            "status": "error",
            "message": "Driver's License verification failed",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)
        
class DocumentUploadView(viewsets.ViewSet):
    permission_classes = [AllowAny]

    def create(self, request):
        user_id = request.data.get('user_id')
        file = request.FILES.get('file')

        if not user_id:
            return Response({"error": "User ID is required."}, status=status.HTTP_400_BAD_REQUEST)

        if not file:
            return Response({"error": "File is required."}, status=status.HTTP_400_BAD_REQUEST)
        data = {
            "user": user_id,
            "file": file
        }
        serializer = DocumentUploadSerializer(data=data)
        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "File uploaded successfully",
                "data": {
                    "status": "Verification pending",
                    "file_url": serializer.instance.file.url,
                    "verify_status": 1
                }
            }, status=status.HTTP_200_OK)
        return Response({
            "status": "error",
            "message": "File upload failed",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)
        
class UnionAssociationViewSet(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]
    
    def list(self, request, user_id=None):
        if not user_id:
            return Response({"error": "user_id is a required parameter."}, status=status.HTTP_400_BAD_REQUEST)

        unions = UnionAssociation.objects.filter(user_id=user_id)
        if not unions.exists():
            return Response({
                "status": "error",
                "message": "No union or association records found for the specified user.",
                "data": []
            }, status=status.HTTP_404_NOT_FOUND)

        serializer = UnionAssociationSerializer(unions, many=True)
        return Response({
            "status": "success",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

    def create(self, request):
        serializer = UnionAssociationSerializer(data=request.data)
        
        if not request.data.get('user'):
            return Response({"error": "User ID is required to create union/association entry."}, status=status.HTTP_400_BAD_REQUEST)

        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Union/Association added successfully",
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)
        
        return Response({
            "status": "error",
            "message": "Union/Association creation failed due to invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def update(self, request, pk=None):
        try:
            union = UnionAssociation.objects.get(pk=pk)
        except UnionAssociation.DoesNotExist:
            return Response({"error": "Union/Association not found."}, status=status.HTTP_404_NOT_FOUND)
        
        serializer = UnionAssociationSerializer(union, data=request.data, partial=True)
        
        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Union/Association updated successfully",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        
        return Response({
            "status": "error",
            "message": "Union/Association update failed due to invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def destroy(self, request, pk=None):
        try:
            union = UnionAssociation.objects.get(pk=pk)
            union.delete()
            return Response({
                "status": "success",
                "message": "Union/Association deleted successfully"
            }, status=status.HTTP_200_OK)
        except UnionAssociation.DoesNotExist:
            return Response({"error": "Union/Association not found."}, status=status.HTTP_404_NOT_FOUND)
        
class HelpOptionsView(APIView):
    permission_classes = [AllowAny]
    
    def get(self, request):
        try:
            options = [
                "Get me more opportunities",
                "Need to network",
                "I am here to hire"
            ]
            return Response({
                "status": "success",
                "message": "Help options retrieved successfully.",
                "data": options
            }, status=status.HTTP_200_OK)
        except Exception as e:
            return Response({
                "status": "error",
                "message": "An error occurred while retrieving help options.",
                "errors": str(e)
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

class ConfirmKYCView(APIView):
    permission_classes = [IsAuthenticated]
    parser_classes = [MultiPartParser, FormParser]  # Enable file uploads

    def post(self, request):
        user_id = request.data.get('user_id')  # Get user_id from the payload
        doc_method = request.data.get('doc_method')
        doc_file = request.FILES.get('doc_file')  # Get the uploaded file

        # Validate user_id
        if not user_id:
            return Response({"error": "user_id is required."}, status=status.HTTP_400_BAD_REQUEST)

        # Validate doc_method
        if doc_method not in ['1', '2', '3']:  # Ensure `doc_method` is a string (from request payload)
            return Response({"error": "Invalid document method. Must be 1, 2, or 3."}, status=status.HTTP_400_BAD_REQUEST)

        # Validate the uploaded file
        if not doc_file:
            return Response({"error": "doc_file is required."}, status=status.HTTP_400_BAD_REQUEST)
        
        if not (doc_file.name.endswith('.jpg') or doc_file.name.endswith('.pdf')):
            return Response({"error": "Invalid file format. Only jpg and pdf are allowed."}, status=status.HTTP_400_BAD_REQUEST)

        try:
            # Save ConfirmKYC object with the uploaded file
            kyc_confirmation = ConfirmKYC.objects.create(
                user=request.user,
                doc_method=doc_method,
                doc_file=doc_file  # Save the uploaded file
            )
            serializer = ConfirmKYCDocumentSerializer(kyc_confirmation)
            return Response({
                "status": "success",
                "message": "KYC details saved successfully.",
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)

        except Exception as e:
            return Response({"error": str(e)}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class GetProfileDeatils(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        user_id = request.query_params.get("userid")  # Use query_params for GET request
        
        if not user_id:
            return Response({
                "status": "error",
                "message": "Invalid or missing user_id."
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            # Assuming get_profile is defined elsewhere to fetch the profile object
            profile = get_profile(user_id)
            print(user_id)
            artizt_network_acc = ArtiztNetworkAccNumber.objects.filter(user=profile.user).first()
            artiztnetwork_accno = artizt_network_acc.account_number if artizt_network_acc else None
            profile_data = {
                "fullname": f"{profile.user.first_name} {profile.user.last_name}" if profile.user else None,
                "profileimg": profile.profile_image.url if profile.profile_image else None,
                "profilecover": profile.cover_image.url if profile.cover_image else None,
                "biography": profile.bio if profile.bio else "",
                "artiztid": profile.user.username,  # Assuming the username is the artist ID
                "artiztnetwork_accno": artiztnetwork_accno,
                "membershiptype": "Artizt Pro",  # Static value for now
                # "mobilenumber": profile.user.phone_number if profile.user.phone_number else "",
                "mailid": profile.user.email,
                "location": profile.location if profile.location else "",
                "dob": profile.date_of_birth.strftime('%d-%B-%Y') if profile.date_of_birth else "",
                "age": profile.age,
                "gender": profile.gender if profile.gender else "",
                "height": str(profile.height) if profile.height else "",
                "weight": f"{profile.weight} kg" if profile.weight else ""
            }
            return Response({
                "status": "success",
                "data": profile_data
            }, status=status.HTTP_200_OK)

        except ObjectDoesNotExist:
            return Response({
                "status": "error",
                "message": "User profile not found."
            }, status=status.HTTP_404_NOT_FOUND)

        except Exception as e:
            return Response({
                "status": "error",
                "message": "Something went wrong. Please try again later.",
                "errors": str(e)
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
        
class StateListView(APIView):
    def get(self, request):
        states = State.objects.all()
        serializer = StateSerializer(states, many=True)
        return Response({"status": "success", "data": serializer.data})

class CityListView(APIView):
    def post(self, request):
        state_id = request.data.get('state_id')
        if not state_id:
            return Response({"status": "error", "message": "State ID is required"}, status=400)
        
        cities = City.objects.filter(state_id=state_id)
        serializer = CitySerializer(cities, many=True)
        return Response({"status": "success", "data": serializer.data})
    
class StateShortNamesListView(APIView):
    def get(self, request):
        states = StateShortNames.objects.all()
        serializer = StateShortNamesSerializer(states, many=True)
        return Response({"status": "success", "data": serializer.data})


class LanguageAPIView(APIView):
    """
    API to return a list of all major languages.
    """
    def get(self, request):
        languages = Language.objects.all()
        if not languages.exists():
            return Response({
                "status": "error",
                "message": "No languages found.",
                "data": []
            }, status=status.HTTP_404_NOT_FOUND)

        serializer = LanguageSerializer(languages, many=True)
        return Response({
            "status": "success",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

class UserLanguageAPIView(APIView):
    def get(self, request, user_id):
        user_languages = UserLanguage.objects.filter(user_id=user_id)
        if not user_languages.exists():
            return Response({
                "status": "error",
                "message": "No known languages found for the specified user.",
                "data": []
            }, status=status.HTTP_404_NOT_FOUND)

        serializer = UserLanguageSerializer(user_languages, many=True)
        return Response({
            "status": "success",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

    def post(self, request):
        user_id = request.data.get('user_id')
        language_name = request.data.get('language_name')
        can_speak = request.data.get('can_speak', False)
        can_write = request.data.get('can_write', False)
        can_read = request.data.get('can_read', False)

        if not user_id or not language_name:
            return Response({
                "status": "error",
                "message": "Both user_id and language_name are required."
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            language = Language.objects.get(name=language_name)
        except Language.DoesNotExist:
            return Response({"error": f"Language '{language_name}' not found."}, status=status.HTTP_404_NOT_FOUND)
        
        if UserLanguage.objects.filter(user_id=user_id, language=language).exists():
            return Response({
                "status": "error",
                "message": f"The language '{language_name}' is already added for this user."
            }, status=status.HTTP_400_BAD_REQUEST)

        # Create or update UserLanguage entry
        user_language, created = UserLanguage.objects.update_or_create(
            user_id=user_id,
            language=language,
            defaults={
                'can_speak': can_speak,
                'can_write': can_write,
                'can_read': can_read
            }
        )
        message = "Known language added successfully." if created else "Known language updated successfully."

        serializer = UserLanguageSerializer(user_language)
        return Response({
            "status": "success",
            "message": message,
            "data": serializer.data
        }, status=status.HTTP_201_CREATED)

    def put(self, request):
        user_id = request.data.get('user_id')
        language_id = request.data.get('language_id')
        can_speak = request.data.get('can_speak', False)
        can_write = request.data.get('can_write', False)
        can_read = request.data.get('can_read', False)

        if not user_id or not language_id:
            return Response({
                "status": "error",
                "message": "Both user_id and language_id are required."
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            user_language = UserLanguage.objects.get(user_id=user_id, language_id=language_id)
            user_language.can_speak = can_speak
            user_language.can_write = can_write
            user_language.can_read = can_read
            user_language.save()

            serializer = UserLanguageSerializer(user_language)
            return Response({
                "status": "success",
                "message": "Known language updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        except UserLanguage.DoesNotExist:
            return Response({
                "status": "error",
                "message": "The specified language record does not exist for this user."
            }, status=status.HTTP_404_NOT_FOUND)
        
class DeleteUserLanguageAPIView(APIView):

    def post(self, request):
        user_id = request.data.get('user_id')
        language_id = request.data.get('language_id')

        if not user_id or not language_id:
            return Response({
                "status": "error",
                "message": "Both user_id and language_id are required."
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            user_language = UserLanguage.objects.get(user_id=user_id, language_id=language_id)
            user_language.delete()
            return Response({
                "status": "success",
                "message": "Known language deleted successfully."
            }, status=status.HTTP_200_OK)
        except UserLanguage.DoesNotExist:
            return Response({
                "status": "error",
                "message": "The specified language record does not exist for this user."
            }, status=status.HTTP_404_NOT_FOUND)


class SearchProfilesAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        keyword = request.data.get("keyword", "")
        industry_ids = request.data.get("industries", [])
        followers = request.data.get("followers", None)
        following = request.data.get("following", None)

        if not keyword or not industry_ids:
            return Response(
                {"status": "error", "message": "Keyword and industries are required."},
                status=status.HTTP_400_BAD_REQUEST,
            )

        try:
            current_user_id = request.user.id
            users = CustomUser.objects.filter(
                Q(full_name__icontains=keyword) &
                Q(profile__selected_industries__id__in=industry_ids)
            ).exclude(id=current_user_id).distinct()

            primary_users = CustomUser.objects.filter(
                Q(full_name__icontains=keyword) &
                Q(profile__selected_primary_industry__id__in=industry_ids)
            ).exclude(id=current_user_id).distinct()
            users = users.union(primary_users)

            # Format data for response
            user_data = []
            for user in users:
                profile = user.profile

                # Fetch primary industry name explicitly from Industry table
                primary_industry = None
                if profile.selected_primary_industry_id:
                    primary_industry = Industry.objects.filter(
                        id=profile.selected_primary_industry_id
                    ).values_list('name', flat=True).first()

                # Get associated industries for the user
                associated_industries = profile.selected_industries.values_list('name', flat=True)  # List of names

                user_data.append({
                    "id": user.id,
                    "username": user.full_name,
                    "profile_image": profile.profile_image.url if profile.profile_image else None,
                    "primary_industry": primary_industry or "N/A",
                    "associated_industries": list(associated_industries),  # All industries associated with the user
                    "follow_status": "false",  # Placeholder for future follow status
                })

            if not user_data:
                return JsonResponse({"status": "error", "message": "No users found"}, status=404)

            return JsonResponse({"status": "success", "data": user_data}, status=200)
        except Exception as e:
            return JsonResponse({"status": "error", "message": str(e)}, status=500)

        return JsonResponse({"status": "error", "message": "Invalid request method"}, status=405)
    
class UpdateProfileStatusAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        user = request.user
        looking_for = request.data.get("looking_for")

        if looking_for not in ['0', '1', '2']:
            return Response(
                {"status": "error", "message": "Invalid status value."},
                status=status.HTTP_400_BAD_REQUEST
            )

        try:
            profile = get_object_or_404(Profile, user=user)
            profile.looking_for = looking_for
            profile.save()

            status_message = "Profile status updated successfully."
            if looking_for == '0':
                status_message = "Profile status removed successfully."

            return Response(
                {"status": "success", "message": status_message},
                status=status.HTTP_200_OK
            )

        except Exception as e:
            return Response(
                {"status": "error", "message": str(e)},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )
    
class ProfileMemberAPIView(APIView):
    def get(self, request, *args, **kwargs):
        user = request.user
        user_id = kwargs.get('user_id')
        profile_members = ProfileMember.objects.filter(owner=user_id)
        serializer = ProfileMemberSerializer(profile_members, many=True)
        return Response({
            "status": "success",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

    def post(self, request, *args, **kwargs):
        owner = request.user
        member_id = request.data.get('member_id')
        role = request.data.get('role')

        if not member_id or not role:
            return Response({
                "status": "error",
                "message": "Both member_id and role are required."
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            member = User.objects.get(id=member_id)
        except User.DoesNotExist:
            return Response({
                "status": "error",
                "message": "The specified member does not exist."
            }, status=status.HTTP_404_NOT_FOUND)

        # Check if the member is already added
        if ProfileMember.objects.filter(owner=owner, member=member).exists():
            return Response({
                "status": "error",
                "message": "This member is already added to your profile."
            }, status=status.HTTP_400_BAD_REQUEST)

        profile_member = ProfileMember.objects.create(owner=owner, member=member, role=role)
        serializer = ProfileMemberSerializer(profile_member)
        return Response({
            "status": "success",
            "message": "Member added successfully.",
            "data": serializer.data
        }, status=status.HTTP_201_CREATED)

    def put(self, request, *args, **kwargs):
        owner = request.user
        member_id = request.data.get('member_id')
        role = request.data.get('role')

        if not member_id or not role:
            return Response({
                "status": "error",
                "message": "Both member_id and role are required."
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            profile_member = ProfileMember.objects.get(owner=owner, member_id=member_id)
            profile_member.role = role
            profile_member.save()

            serializer = ProfileMemberSerializer(profile_member)
            return Response({
                "status": "success",
                "message": "Role updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        except ProfileMember.DoesNotExist:
            return Response({
                "status": "error",
                "message": "The specified member does not exist in your profile."
            }, status=status.HTTP_404_NOT_FOUND)

    def delete(self, request, *args, **kwargs):

        owner = request.user
        # member_id = request.data.get('member_id')
        member_id = kwargs.get('member_id') 

        if not member_id:
            return Response({
                "status": "error",
                "message": "member_id is required."
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            profile_member = ProfileMember.objects.get(owner=owner, member_id=member_id)
            profile_member.delete()
            return Response({
                "status": "success",
                "message": "Member removed successfully."
            }, status=status.HTTP_200_OK)
        except ProfileMember.DoesNotExist:
            return Response({
                "status": "error",
                "message": "The specified member does not exist in your profile."
            }, status=status.HTTP_404_NOT_FOUND)
    
class StoreIndustryAPIView(APIView):
    authentication_classes = []
    permission_classes = [AllowAny]
    def post(self, request):
        try:
            # Extract the data from the request
            data = request.data
            name = data.get("name")
            image = request.FILES.get("image")

            # Validation: check if both name and image are provided
            if not name or not image:
                return Response({
                    "status": "error",
                    "message": "Both 'name' and 'image' are required."
                }, status=status.HTTP_400_BAD_REQUEST)
            
            if Industry.objects.filter(name=name).exists():
                return Response({
                    "status": "error",
                    "message": f"An Industry with the name '{name}' already exists."
                }, status=status.HTTP_400_BAD_REQUEST)

            # Create and save the industry instance
            industry = Industry.objects.create(name=name, image=image)

            # Construct the image URL using the Django media URL settings
            image_url = request.build_absolute_uri(industry.image.url) if industry.image else None

            # Return success response with industry details and image URL
            return Response({
                "status": "success",
                "message": "Industry created successfully.",
                "data": {
                    "id": industry.id,
                    "name": industry.name,
                    "image_url": image_url
                }
            }, status=status.HTTP_201_CREATED)

        except Exception as e:
            # In case of any exception, return a server error
            return Response({
                "status": "error",
                "message": f"An error occurred: {str(e)}"
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
        
class StoreSkillAPIView(APIView):
    authentication_classes = []
    permission_classes = [AllowAny]
    def post(self, request):
        try:
            # Extract data from request
            name = request.data.get("name")
            image = request.FILES.get("image")  # Expecting image file, not image_url
            industry_id = request.data.get("industry_id")

            # Validate required fields
            if not name or not image or not industry_id:
                return Response({
                    "status": "error",
                    "message": "name, image, and industry_id are required."
                }, status=status.HTTP_400_BAD_REQUEST)
            
            if Skill.objects.filter(name=name, industry_id=industry_id).exists():
                return Response({
                    "status": "error",
                    "message": f"A Skill with the name '{name}' already exists in this industry."
                }, status=status.HTTP_400_BAD_REQUEST)

            # Check if the industry exists
            try:
                industry = Industry.objects.get(id=industry_id)
            except Industry.DoesNotExist:
                return Response({
                    "status": "error",
                    "message": "Industry with the provided ID does not exist."
                }, status=status.HTTP_404_NOT_FOUND)

            # Create and save the skill instance
            skill = Skill.objects.create(name=name, image=image, industry=industry)

            # Construct the image URL
            image_url = request.build_absolute_uri(skill.image.url) if skill.image else None

            # Return success response with skill details
            return Response({
                "status": "success",
                "message": "Skill created successfully.",
                "data": {
                    "id": skill.id,
                    "name": skill.name,
                    "image_url": image_url,
                    "industry": {"id": industry.id, "name": industry.name}
                }
            }, status=status.HTTP_201_CREATED)

        except Exception as e:
            # Handle any unexpected errors
            return Response({
                "status": "error",
                "message": f"An error occurred: {str(e)}"
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
        
class SyncMobileNumber(APIView):
    def post(self, request):
        phone_numbers = request.data.get('phone_numbers', [])
        
        if not isinstance(phone_numbers, list) or not phone_numbers:
            return Response(
                {"error": "Invalid input. Please provide a list of phone numbers."},
                status=status.HTTP_400_BAD_REQUEST
            )
        
        # Fetch users based on phone numbers
        users = CustomUser.objects.filter(mobile_number__in=phone_numbers)
        results = []
        
        for user in users:
            try:
                profile = Profile.objects.get(user=user)
                am_I_following = Follow.objects.filter(follower_id = request.user , following_id = user).exists()
                print(am_I_following)
                results.append({
                    "user_id": user.id,
                    "username": user.username,
                    "email": user.mobile_or_email,
                    "mobile_number": user.mobile_number,
                    "full_name":user.full_name,
                    "user_type": profile.get_user_type_display(),
                    "bio": profile.bio,
                    "age": profile.age,
                    "gender": profile.gender,
                    "location": profile.location,
                    "is_verified": profile.is_verified,
                    "profile_image": profile.profile_image.url if profile.profile_image else None,
                    "am_I_following":am_I_following,
                })
            except Profile.DoesNotExist:
                results.append({
                    "user_id": user.id,
                    "username": user.username,
                    "email": user.mobile_or_email,
                    "mobile_number": user.mobile_number,
                    "full_name":user.full_name,
                    "profile": None
                })
        
        return Response({"results": results}, status=status.HTTP_200_OK)